;Written for AT90S2313 @ 1.8432 MHz

;===============================================================================

;      Converter for serial control of HD44780 and compatible LC displays

;===============================================================================

;The LCD can only be controlled in 4 bit mode. The bits of the sent bytes are
;used as follows:
;  0..3  -  D4..D7
;  4     -  RS
;  5     -  R/W' (currently, only write operations can be performed)
;  6     -  Controller selelct (0 equals first controller)
;The contrast of the LCD can be set by sending a byte with bit 7 set and a
;value between 0 and 127 as a contrast value (0...5V on the PWM output)
;stored in the other 7 bits.

;The outputs of the AVR are used as follows:
;  PD0     -  RS232 input (RxD)
;  PD1     -  RS232 output (TxD, currently not needed)
;  (Note that the signals are inverted compared to RS232)
;  PD6     -  LCD E2 (second controller, if present)

;  PB0     -  LCD E1 (first controller)
;  PB1     -  LCD RS
;  PB2     -  LCD R/W' (currently not used, always 0)
;  PB3     -  LCD V_LCD (use resistor and buffer capacitor!)
;  PB4..7  -  LCD D4..D7


;Version 1.0 finished 24.04.2004

;===============================================================================

.include "..\2313def.inc"
.def	temp	= r16
.def	out1	= r17
.def	out2	= r18
.def	w1	= r19
.def	w2	= r20

.macro WAIT
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
.endmacro

;===============================================================================

.cseg
.org $0000
	rjmp	start
.org $0007
	push	temp
	in	temp,	SREG
	push	temp
	
	in	temp,	UDR
	sbrc	temp,	7
	rjmp	setpwm
		
	mov	out1,	temp
	lsl	out1
	lsl	out1
	lsl	out1
	lsl	out1
	sbrc	temp,	4	;RS
	ori	out1,	0x02
	;sbis	temp,	5	;R/W'   --> only write in this version! <--
	;ori	out1,	0x04
	
	out	PORTB,	out1
	nop
	WAIT
	sbrs	temp,	6	;E1
	sbi	PORTB,	0
	sbrc	temp,	6	;E2
	sbi	PORTD,	6
	WAIT
	cbi	PORTB,	0
	cbi	PORTD,	6
	WAIT

return:
	out	UDR,	temp		;for test purposes, remove in next version!

	pop	temp
	out	SREG,	temp
	pop	temp
	reti
	
setpwm:
	lsl	temp
	out	OCR1AL,	temp
	rjmp	return

;===============================================================================

start:
	ldi	temp,	low(RAMEND)
	out	SPL,	temp
	ldi	temp,	0b11111111
	out	DDRB,	temp
	ldi	temp,	0b01111110
	out	DDRD,	temp
	ldi	temp,	0b00000000
	out	PORTB,	temp
	ldi	temp,	0b11111101
	out	PORTD,	temp
	
	ldi	temp,	0b10000001	;set PWM to sysclk/8, 8 bit, non-inv, enabled, 0V
	out	TCCR1A,	temp
	ldi	temp,	0b00000010
	out	TCCR1B,	temp
	ldi	temp,	0
	out	OCR1AL,	temp	
	
	ldi	temp,	0b10011000	;set UART to 9600 Baud, Rx+Tx, Rx_Int on
	out	UCR,	temp
	ldi	temp,	11
	out	UBRR,	temp
	sei
	
loop:
	rjmp	loop

;===============================================================================
